/***
* 		mono.c -- b&p pad tool to gate overlapping notes in a clip
*  	 1991 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE    | WHAT
============================================================================
	 2 Dec 91 | created
	11 Dec 91 | new icon
   19 Jan 93 | massaged for sas/c v6.0
	15 Nov 93 | new window title
	15 Nov 96 | back to small icon; regargs; amiga.lib
*/

#define THIS_TOOL_NAME "MONO"

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#include <string.h>
//#include <stdio.h>

#include "bars.h"
#include "barsMacros.h"
#include "bplib.h"
#include "monow.h"
#include "proto/mono_protos.h"
#include "/icons/mono.h"

static const UBYTE versiontag[] = "$VER: Mono 37.13 (15.11.96)";

struct MonoTool {		/* a Tool structure for the tool */
	struct Tool tool;
	UBYTE gate;
};

#define THIS_TOOL_SIZE sizeof(struct MonoTool) /* for ToolMaster */

extern struct Functions *functions;

/*==========================================================================
* init_tool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: inits tool
*=========================================================================*/

static __inline void init_tool(tool)

struct MonoTool *tool;

{
	tool->gate = 90;
	tool->tool.touched |= TOUCH_INIT;
}

/*==========================================================================
* hilopercent
*
*  INPUT: pointers to window and gadget, gadget value
* OUTPUT: none
* RETURN: gadget value
*
* EFFECT: display the pot value of the gadget as a percentage.  
*=========================================================================*/
/*
#define XOffset 43 /* relative to g->LeftEdge + g->Width */
#define YOffset 6  /* relative to g->TopEdge */
*/
#define TEXTCOLOR 1

static long __saveds __stdargs hilopercent(w,g,h)

struct Window *w;
struct Gadget *g;
long h;

{
	char text[5];

	if (h==0) h=1;
	SetRPort(w->RPort,g->LeftEdge+g->Width+XOffset,g->TopEdge+YOffset,TEXTCOLOR,0,JAM2);
	sprintf(text,"%3ld%%",h);
	Text(w->RPort,text,4);
	return(h);
}

#define tw tool->tool.window

/*==========================================================================
* edittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: twiddles with a pot to get a gate value
*=========================================================================*/

static void __saveds __stdargs edittool(tool)

struct MonoTool *tool;

{
	struct IntuiMessage *message;
	struct Gadget *gadget;
	long class,
	     code;
	struct NewWindow *nw;

	if (!tool->tool.touched) init_tool(tool);
	nw=NewOpenEmbossedWindow(&NewWindowStructure1,&tool->tool,0,WTITLE);
	if (!nw) return;
	SetMenuStrip(tw,&Menu1);
	(*functions->FatEmbossedPropOn)(tw,1,2,3,hilopercent,101,1);

	for (;;) {
		(*functions->ModifyEmbossedProp)(tw,1,tool->gate,0,0,0,0,0);
		(*functions->DrawEmbossedProp)(tw,1);
		message = (struct IntuiMessage *)(*functions->GetIntuiMessage)(tw);
		class = message->Class;
		code = message->Code;
		gadget = (struct Gadget *)message->IAddress;
		ReplyMsg((struct Message *)message);
		class = (*functions->SystemGadgets)(tw,class,gadget,code);

		if (class == CLOSEWINDOW) break;

		else if (class == GADGETDOWN) {
			switch (gadget->GadgetID) {
				case 1 :
					tool->gate = (*functions->DragEmbossedProp)(tw,1);
					break;
				case 2 :
					tool->gate = (*functions->ShiftEmbossedProp)(tw,1,-1,0);
					break;
				case 3 :
					tool->gate = (*functions->ShiftEmbossedProp)(tw,1,1,0);
					break;
			}
		}
	}
	(*functions->FatEmbossedPropOff)(tw,1,2,3);
	NewCloseEmbossedWindow(&tool->tool,nw);
}

/*==========================================================================
* processlist
*
*  INPUT: pointers to tool and list to process, start and stop times
* RETURN: pointer to list
*
* EFFECT: chops durations of note events that overlap a subsequent event
*=========================================================================*/

static struct Event * __saveds __stdargs processlist(tool,list,start,stop)

struct MonoTool *tool;
struct NoteEvent *list;
long start, stop;

{
	register struct NoteEvent *note = list;
	register struct NoteEvent *mark;

	if (!(tool->tool.touched)) init_tool(tool);

	if (list) {
		/* skip non-note events */
		while ((note->status != MIDI_NOTEON) && (note->time <= stop))
			note = note->next;

		while (note && (note->time <= stop) ) {
			if (note->status = MIDI_NOTEON) {
				mark = note;
				while (note->time == mark->time) { /* find next event time */
					note = note->next;
				}
				while (mark != note) {
					if ( (mark->time + mark->duration) > note->time) {
						mark->duration = (note->time-mark->time) * tool->gate / 100;
					}
					mark = mark->next;
				}
			}
			else note = note->next;
		}
	}
	return((struct Event *)list);
}

/*==========================================================================
* inittoolmaster
*
*  INPUT: none
* RETURN: pointer to initialized ToolMaster
*
* EFFECT: inits
*=========================================================================*/

static struct ToolMaster master;

struct ToolMaster * __saveds __stdargs inittoolmaster()

{
	memset((char *)&master,0,sizeof(struct ToolMaster));
	master.toolid = THIS_TOOL_ID;
	master.image = &MonoIcon;
	strcpy(master.name,WTITLE);
	master.edittool = edittool;
	master.processlist = processlist;
	master.toolsize = THIS_TOOL_SIZE;
	master.tooltype = TOOL_NORMAL;

	return((struct ToolMaster *)&master);
}
